#include <sourcemod>
#include <sdktools>

#pragma semicolon 1

public Plugin:myinfo = 
{
	name = "Simple Weapon Restriction",
	author = "WhiteWolf",
	description = "Weapon Restriction",
	version = "1.6",
	url = "http://www.whitewolf.us"
};
/* Credits:
	devicenull - Original code + understanding some things in pawn
dalto - Sell! config/EquipAvailableWeapon used with permissions
*/

#define MAX_RESTRICT 32

#define RESTRICT_BUY 0
#define RESTRICT_ROUNDSTART 1

//The list of restricted weapons
new String:g_buyRestricted[MAX_RESTRICT][64];
new g_curBuyRestricted;

new String:g_roundRestricted[MAX_RESTRICT][64];
new g_curRoundRestricted;

//Offset to the list of weapons this player has
new off_myWeapons;

//Offset to the players currently active weapon
new off_activeWeapon;


new Handle:sm_enable_restrict;
new Handle:sm_enable_round_restrict;
new Handle:sm_roundcheck_delay;

new Handle:hGameConf = INVALID_HANDLE;
new Handle:hEquipWeapon = INVALID_HANDLE;

public OnPluginStart() {
	HookEvent("item_pickup", ev_ItemPickup, EventHookMode_Post);
	HookEvent("player_spawn", EventPlayerSpawn);

	RegAdminCmd("sm_restrict", cmdRestrict, ADMFLAG_CONFIG, "", "Restrict a weapon.", 0);
	RegAdminCmd("sm_round_restrict", cmdRestrict, ADMFLAG_CONFIG, "", "Restrict a weapon from being retained at the start of a round.");

	RegAdminCmd("sm_unrestrict", cmdUnRestrict, ADMFLAG_CONFIG, "", "Unrestrict a weapon.");
	RegAdminCmd("sm_round_unrestrict", cmdUnRestrict, ADMFLAG_CONFIG, "", "Unrestrict a round start restricted weapon.");

	RegAdminCmd("sm_restrict_list", cmdRestrictList, ADMFLAG_GENERIC, "", "Lists all the weapon restrictions in place.");

	sm_enable_restrict = CreateConVar("sm_enable_restrict", "1", "Enable weapon restriction checks.");
	sm_enable_round_restrict = CreateConVar("sm_enable_round_restrict", "1", "Enable weapon restriction checks.");
	sm_roundcheck_delay = CreateConVar("sm_roundcheck_delay", "0.20", "Delay after roundstart for weapons checking.");

	/* Conf file from Sell! */
	hGameConf = LoadGameConfigFile("wwrestrict.games");
	if(hGameConf == INVALID_HANDLE) {
		SetFailState("gamedata/wwrestrict.games.txt not loadable");
	}

	off_myWeapons = FindSendPropOffs("CAI_BaseNPC", "m_hMyWeapons");
	off_activeWeapon = FindSendPropOffs("CAI_BaseNPC", "m_hActiveWeapon");
	if(off_myWeapons < 1 || off_activeWeapon < 1) {
		SetFailState("Could not find offsets required for restriction to work.");
	}

	/* From Sell! */
	StartPrepSDKCall(SDKCall_Player);
	PrepSDKCall_SetFromConf(hGameConf, SDKConf_Virtual, "Weapon_Equip");
	PrepSDKCall_AddParameter(SDKType_CBaseEntity, SDKPass_Pointer);
	hEquipWeapon = EndPrepSDKCall();

	//We're done with hGameConf -- ^Bugs^ hinted that it might be leaking
	CloseHandle(hGameConf);

}

/*******************************************************************************
	Console Command Handlers
********************************************************************************/

public Action:cmdRestrictList(client, args) {
	ReplyToCommand(client, "Buy Restrictions (%d entries)", g_curBuyRestricted);

	for(new i = 0; i < g_curBuyRestricted; i++) {
		ReplyToCommand(client, "%s", g_buyRestricted[i]);
	}

	ReplyToCommand(client, "");
	ReplyToCommand(client, "Round Start Restrictions (%d entries)", g_curRoundRestricted);

	for(new i =0; i < g_curRoundRestricted; i++) {
		ReplyToCommand(client, "%s", g_roundRestricted[i]);
	}
	return Plugin_Handled;
}

public Action:cmdRestrict(client, args) {
	//first let's see how we're called
	decl String:cmdname[32];
	GetCmdArg(0, cmdname, sizeof(cmdname));

	if (args != 1) {
		ReplyToCommand(client, "Usage: %s weapon", cmdname);
		return Plugin_Handled;
	}

	decl String:wepname[64];
	GetCmdArg(1, wepname, 64);

	new bool:restricted;
	if(strcmp(cmdname, "sm_round_restrict", false) == 0) {
		restricted = isWeaponRestricted(wepname, g_roundRestricted, g_curRoundRestricted);
		if(!restricted) {
			if(g_curRoundRestricted == MAX_RESTRICT) {
				ReplyToCommand(client, "Cannot round restrict, maximum round restricts reached. (%d)", MAX_RESTRICT);
			} else if(RestrictWeapon(wepname, g_roundRestricted, g_curRoundRestricted)) {
				ReplyToCommand(client, "%s was round restricted successfully.", wepname);
			} else {
				ReplyToCommand(client, "There was an error round restricting %s", wepname);
			}
		} else {
			ReplyToCommand(client, "%s is already round restricted.", wepname);
			return Plugin_Handled;
		}
	} else {
		restricted = isWeaponRestricted(wepname, g_buyRestricted, g_curBuyRestricted);
		if(!restricted) {
			if(g_curBuyRestricted == MAX_RESTRICT) {
				ReplyToCommand(client, "Cannot buy restrict, maximum buy restricts reached. (%d)", MAX_RESTRICT);
			} else if(RestrictWeapon(wepname, g_buyRestricted, g_curBuyRestricted)) {
				ReplyToCommand(client, "%s was buy restricted successfully.", wepname);
			} else {
				ReplyToCommand(client, "There was an error buy restricting %s", wepname);
			}
		} else {
			ReplyToCommand(client, "%s is already buy restricted.", wepname);
			return Plugin_Handled;
		}
	}

	return Plugin_Handled;
}

public Action:cmdUnRestrict(client, args) {
	//first let's see how we're called
	decl String:cmdname[32];
	GetCmdArg(0, cmdname, sizeof(cmdname));

	decl String:wepname[64];
	GetCmdArg(1, wepname, 64);

	if (args != 1) {
		ReplyToCommand(client, "Usage: %s weapon", cmdname);
		return Plugin_Handled;
	}

	new bool:unrestricted = false;
	if(strcmp(cmdname, "sm_round_unrestrict", false) == 0) {
		unrestricted = UnRestrictWeapon(wepname, g_roundRestricted, g_curRoundRestricted);
	} else {
		unrestricted = UnRestrictWeapon(wepname, g_buyRestricted, g_curBuyRestricted);
	}

	if(unrestricted) {
		ReplyToCommand(client, "%s was unrestricted.", wepname);
	} else {
		ReplyToCommand(client, "%s was not unrestricted, probably not restricted.", wepname);
	}
	return Plugin_Handled;
}

public bool:isValidHumanClient(client) {
	if(client > 0 && IsClientConnected(client) && IsClientInGame(client) && !IsFakeClient(client)) {
		return true;
	}
	return false;
}

public Action:CheckPlayer(Handle:timer, any:client) {
	if(!isValidHumanClient(client) || IsClientObserver(client)) {
		return Plugin_Stop;
	}

	PrintToChat(client, "Checking you for restricted round items...");
	RemoveWeapons(client, g_roundRestricted, g_curRoundRestricted, true);		
	return Plugin_Stop;
}

/*******************************************************************************
	Events
********************************************************************************/
public Action:EventPlayerSpawn(Handle:event, const String:name[], bool:dontBroadcast) {
	new playerId = GetEventInt(event, "userid");
	if(GetConVarInt(sm_enable_round_restrict) == 1) {
		CreateTimer(GetConVarFloat(sm_roundcheck_delay), CheckPlayer, GetClientOfUserId(playerId));
	}
	return Plugin_Handled;
}

public Action:ev_ItemPickup(Handle:event, const String:name[], bool:dontBroadcast) {
	new String:item[64];
	new enabled;
	enabled = GetConVarInt(sm_enable_restrict);

	if(enabled == 0) {
		return Plugin_Handled;
	}

	GetEventString(event,"item",item,64);
	for (new i=0;i<g_curBuyRestricted;i++) {
		//Skip the weapon_ prefix
		if (strcmp(item,g_buyRestricted[i][7],false) == 0) {
			// Player has a forbidden weapon
			new client = GetClientOfUserId(GetEventInt(event, "userid"));
			RemoveWeapons(client, g_buyRestricted, g_curBuyRestricted);
			break;
		}
	}
	return Plugin_Handled;
}

/*******************************************************************************
	Internal Restriction Management
********************************************************************************/

public bool:isWeaponRestricted(String:clsname[], String:restrictedItems[][64], &curRestricted) {
	for(new i = 0; i<curRestricted; i++) {
		if(strcmp(clsname, restrictedItems[i], false) == 0) {
			return true;
		}
	}
	return false;
}

public RestrictWeapon(String:weapon[], String:restrictedItems[][64], &curRestricted) {
	if(curRestricted == MAX_RESTRICT) {
		//ReplyToCommand(client, "Can't add buy restriction, max restrictions reached for category. (%d)", MAX_RESTRICT);
		return false;
	}

	strcopy(restrictedItems[curRestricted], 64, weapon);
	++curRestricted;
	return true;
}

public bool:UnRestrictWeapon(String:weapon[], String:restrictedItems[][64], &curRestricted) {
	new found;

	for (new i=0;i<curRestricted;++i) {
		if (strcmp(weapon,restrictedItems[i],false) == 0) {
			//Found it! Shift all others down
			for (new j=i+1;j<curRestricted;++j) {
				strcopy(restrictedItems[j-1], 64, restrictedItems[j]);
			}
			found++;
			--curRestricted;
			break;
		}
	}
	if(found == 1) {
		return true;
	} else {
		return false;
	}
}

/*******************************************************************************
	Client Management
********************************************************************************/

RemoveWeapons(client, String:restrictedItems[][64], itemCount, bool:removeEdict=false) {
	if(!isValidHumanClient(client) || IsClientObserver(client)) {
	return;
	}

	if(itemCount == 0) {
		return;
	}
	decl activeWep;
	activeWep = GetEntDataEnt(client, off_activeWeapon);

	new entindex;
	new String:clsname[64];
	for (new i=0; i<(4*12); i+=4) { //each weapon takes 4 bytes, we want to look at 12 slots worth
		//Get the entity index for the weapon in this slot
		entindex = GetEntDataEnt(client, off_myWeapons + i);
		if (entindex != 0) {
			GetEdictClassname(entindex, clsname, 64);
			for (new j=0;j<itemCount;++j) {
				//Is this one of the restricted weapons?
				if (strcmp(clsname, restrictedItems[j], false) == 0) {
					RemovePlayerItem(client, entindex); //remove the item from the player
					if(entindex == activeWep) {
						EquipAvailableWeapon(client);
					}
					if(removeEdict) {
						RemoveEdict(entindex); // Crash if we're dealing with item pickup... but not at round start
					}
					break;
				}//ei
			}//ef
		}//ei
	}//ef
}

public EquipAvailableWeapon(client) {
	// Find a new weapon to equip
	new pos = 0;
	new weaponEntity = -1;
	do {
		weaponEntity = GetPlayerWeaponSlot(client, pos);
		pos++;
	} while(weaponEntity == -1 && pos < 5);
	if(weaponEntity != -1) {
		SDKCall(hEquipWeapon, client, weaponEntity);
	}
}